# string-similarity-score

Finds degree of similarity (Percentage or score) between two strings, based on [Dice's Coefficient](http://en.wikipedia.org/wiki/S%C3%B8rensen%E2%80%93Dice_coefficient), which is mostly better than [Levenshtein distance](http://en.wikipedia.org/wiki/Levenshtein_distance).

## Table of Contents

- [string-similarity](#string-similarity)
  - [Table of Contents](#table-of-contents)
  - [Usage](#usage)
    - [For Node.js](#for-nodejs)
  - [API](#api)
    - [compareTwoStrings(string1, string2)](#comparetwostringsstring1-string2)
      - [Arguments](#arguments)
      - [Returns](#returns)
      - [Examples](#examples)
    - [findBestMatch(mainString, targetStrings)](#findbestmatchmainstring-targetstrings)
      - [Arguments](#arguments-1)
      - [Returns](#returns-1)
      - [Examples](#examples-1)

## Usage

#### For Node.js

Install using:

```shell
npm install string-similarity-score --save
```

In your code:

```javascript
var stringSimilarity = require("string-similarity-score");

var similarity = stringSimilarity.compareTwoStrings("healed", "sealed");

var matches = stringSimilarity.findBestMatch("healed", [
  "edward",
  "sealed",
  "theatre",
]);
```

## API

The package contains two methods:

### compareTwoStrings(string1, string2)

Returns a fraction between 0 and 1, which indicates the degree of similarity between the two strings. 0 indicates completely different strings, 100 indicates identical strings. The comparison is case-sensitive.

##### Arguments

1. string1 (string): The first string
2. string2 (string): The second string

Order does not make a difference.

##### Returns

(number): A Percentage from 0 to 100, both inclusive. Higher number indicates more similarity.

##### Examples

```javascript
stringSimilarity.compareTwoStrings("healed", "sealed");
// → 80

stringSimilarity.compareTwoStrings(
  "Olive-green table for sale, in extremely good condition.",
  "For sale: table in very good  condition, olive green in colour."
);
// → 60

stringSimilarity.compareTwoStrings(
  "Olive-green table for sale, in extremely good condition.",
  "For sale: green Subaru Impreza, 210,000 miles"
);
// → 25

stringSimilarity.compareTwoStrings(
  "Olive-green table for sale, in extremely good condition.",
  "Wanted: mountain bike with at least 21 gears."
);
// → 14
```

### findBestMatch(mainString, targetStrings)

Compares `mainString` against each string in `targetStrings`.

##### Arguments

1. mainString (string): The string to match each target string against.
2. targetStrings (Array): Each string in this array will be matched against the main string.

##### Returns

(Object): An object with a `ratings` property, which gives a similarity rating for each target string, a `bestMatch` property, which specifies which target string was most similar to the main string, and a `bestMatchIndex` property, which specifies the index of the bestMatch in the targetStrings array.

##### Examples

```javascript
stringSimilarity.findBestMatch('Olive-green table for sale, in extremely good condition.', [
  'For sale: green Subaru Impreza, 210,000 miles',
  'For sale: table in very good condition, olive green in colour.',
  'Wanted: mountain bike with at least 21 gears.'
]);
// →
{ ratings:
   [ { target: 'For sale: green Subaru Impreza, 210,000 miles',
       rating: 25 },
     { target: 'For sale: table in very good condition, olive green in colour.',
       rating: 60 },
     { target: 'Wanted: mountain bike with at least 21 gears.',
       rating: 14 } ],
  bestMatch:
   { target: 'For sale: table in very good condition, olive green in colour.',
     rating: 60 },
  bestMatchIndex: 1
}
```
